<?php

/**
 * Form input for adding images from Wikipedia or Wikimedia Commons.
 * 
 * @since 0.1
 * 
 * @file InstantImageInput.php
 * @ingroup SFFormInput
 * @ingroup SII
 * 
 * @licence GNU GPL v3+
 * @author Jeroen De Dauw < jeroendedauw@gmail.com >
 */
class InstantImageInput extends SFFormInput {
	
	public static function getName() {
		return 'instantimage';
	}

	public static function getDefaultPropTypes() {
		return array();
	}

	public static function getOtherPropTypesHandled() {
		return array( '_txt', '_wpg' );
	}

	public static function getDefaultPropTypeLists() {
		return array();
	}
	
	public static function getOtherPropTypeListsHandled() {
		return array();
	}
	
	/**
	 * Returns the HTML code to be included in the output page for this input.
	 */
	public function getHtmlText() {
		return self::getHTML(
			$this->mCurrentValue,
			$this->mInputName,
			$this->mIsMandatory,
			$this->mIsDisabled,
			$this->mOtherArgs
		);
	}
	
	public static function getParameters() {
		$params = parent::getParameters();
		
		$params[] = array(
			'name' => 'type',
			'type' => 'str',
			'description' => wfMessage( 'sii-imageinput-type' )->text()
		); // page
		
		$params[] = array(
			'name' => 'hide',
			'type' => 'bool',
			'description' => wfMessage( 'sii-imageinput-hide' )->text()
		); // false
		
		$params[] = array(
			'name' => 'width',
			'type' => 'int',
			'description' => wfMessage( 'sii-imageinput-width' )->text()
		); // 200
		
		$params[] = array(
			'name' => 'showdefault',
			'type' => 'bool',
			'description' => wfMessage( 'sii-imageinput-showdefault' )->text()
		); // true
		
		return $params;
	}
	
	public static function getHTML( $cur_value, $input_name, $is_mandatory, $is_disabled, $other_args ) {
		global $wgOut;
		$html = '';
		
		if ( !array_key_exists( 'page', $other_args ) ) {
			$other_args['type'] = 'page';
		}
		
		$showDefault = !array_key_exists( 'showdefault', $other_args ) || $other_args['showdefault'] === 'yes';
		
		if ( $showDefault && !array_key_exists( 'default', $other_args ) ) {
			$other_args['default'] = SIISettings::get( 'defaultImage' );
		}
		
		$defImg = $other_args['default'];
		
		$noImage = is_null( $cur_value ) || trim( $cur_value ) === '';
		$showInForm = !array_key_exists( 'hide', $other_args ) || trim( $other_args['hide'] ) === 'no';
		
		$width = array_key_exists( 'width', $other_args ) ? $other_args['width'] : SIISettings::get( 'defaultWidth' );
		
		if ( $showInForm && !$noImage ) {
			if ( $noImage ) {
				$html .= wfMessage( 'sii-imageinput-loading' )->escaped();
			}
			else {
				global $wgParser;
				$html .= $wgParser->recursiveTagParse( '[[' . $cur_value . '|' . $width . 'px ]]' );				
			}
		}
		
		if ( $noImage || $cur_value === $defImg ) {
			$wgOut->addModules( 'sii.image' );
			$cur_value = $defImg;
			
			$args = array(
				'class' => 'instantImage',
				'data-image-type' => $other_args['type'],
				'data-input-name' => $input_name,
				'data-image-width' => $width,
			);
			
			if ( $other_args['type'] == 'page' ) {
				$args['data-image-name'] = self::getPage();
			}
			
			$html = Html::rawElement(
				'div',
				$args,
				$html
			);
		}
		
		return $html . Html::input( $input_name, $cur_value, 'hidden' );
	}
	
	protected static function getPage() {
		global $wgParser;
		return $wgParser->getTitle()->getFullText();
	}
	
}
